@@ -27,9 +27,8 @@
 Test the llidl module
 """
 
-
 import datetime
-import StringIO
+import io
 import unittest
 
 from llbase import llidl
@@ -66,7 +65,7 @@ def _binary():
     """
     Return a llsd.binary object for testing.
     """
-    return llsd.binary("\0x01\x02\x03")
+    return llsd.binary(b"\0x01\x02\x03")
 
 class LLIDLTypeTests(unittest.TestCase):
     """
@@ -137,14 +136,22 @@ class LLIDLTypeTests(unittest.TestCase):
         self.assert_(v.match(0.0))
         self.assert_(v.match(10.0))
         self.assert_(v.incompatible(3.14))
-        self.assert_(v.incompatible(6.02e23))
+        if llsd.PY2:
+            # In Python 2, this would require a 'long', but it fits in a
+            # Python 3 'int' and hence is not incompatible().
+            self.assert_(v.incompatible(6.02e23))
+        else:
+            self.assert_(v.match(6.02e23))
         self.assert_(v.has_defaulted(""))
         self.assert_(v.match("0"))
         self.assert_(v.match("1"))
         self.assert_(v.match("0.0"))
         self.assert_(v.match("10.0"))
         self.assert_(v.incompatible("3.14"))
-        self.assert_(v.incompatible("6.02e23"))
+        if llsd.PY2:
+            self.assert_(v.incompatible("6.02e23"))
+        else:
+            self.assert_(v.match("6.02e23"))
         self.assert_(v.incompatible("blob"))
         self.assert_(v.incompatible(_dateToday()))
         self.assert_(v.incompatible(_uuid()))
@@ -1054,7 +1061,7 @@ class LLIDLParseSuiteTests(unittest.TestCase):
         try:
             v = llidl.parse_suite(s)
             self.fail('Incorrectly parsed "%s" as valid' % s)
-        except llidl.ParseError, e:
+        except llidl.ParseError as e:
             if msg and msg not in str(e):
                 self.fail('Missing "%s" in "%s"' % (msg, str(e)))
             if line:
@@ -1665,14 +1672,14 @@ class LLIDLParsingTests(unittest.TestCase):
     def good_parse_value(self, s):
         try:
             v = llidl.parse_value(s)
-        except llidl.ParseError, e:
+        except llidl.ParseError as e:
             self.fail('Failed parsing "%s", %s' % (s, str(e)))
 
     def bad_parse_value(self, s, msg=None, line=None, char=None):
         try:
             v = llidl.parse_value(s)
             self.fail('Incorrectly parsed "%s" as valid' % s)
-        except llidl.ParseError, e:
+        except llidl.ParseError as e:
             if msg and msg not in str(e):
                 self.fail('Mising "%s" in "%s"' % (msg, str(e)))
             if line:
@@ -1921,7 +1928,7 @@ class LLIDLExceptionTests(unittest.TestCase):
         try:
             s.match_request('some_api', {'id': 'bob'})
             self.fail("should have raised an exception")
-        except Exception, me:
+        except Exception as me:
             self.assertEqual(str(me),
                 "Resource name 'some_api' not found in suite.")
             
@@ -1937,13 +1944,13 @@ class LLIDLReportingTests(unittest.TestCase):
 
 class LLIDLFileTests(unittest.TestCase):
     def test_parse_value_from_file(self):
-        file = StringIO.StringIO("[ int, int ]")
+        file = io.StringIO(u"[ int, int ]")
         v = llidl.parse_value(file)
         self.assert_(v.match([1,2]))
         self.assert_(v.incompatible(["one", "two"]))
         
     def test_parse_suite_from_file(self):
-        file = StringIO.StringIO(""";test suite
+        file = io.StringIO(u""";test suite
 %% agent/name
 -> { agent_id: uuid }
 <- { first: string, last: string }
